package IPTC::Factory;

use version; our $VERSION = qv('0.1_1');
use strict;

use Util::Logfile;
use IPTC::ExifToolReader;
use IPTC::ExifToolWriter;
use IPTC::ExifUtilsReader;
use IPTC::ExifUtilsWriter;

sub create_reader {
    _check_globals();
    _print_info("Lesen");

    return $main::USE_EXIFTOOL
        ? IPTC::ExifToolReader->new()
        : IPTC::ExifUtilsReader->new($main::EU_OPTION_PRAEFIX);
}

sub create_writer {
    _check_globals();
    _print_info("Schreiben");

    return $main::USE_EXIFTOOL
        ? IPTC::ExifToolWriter->new()
        : IPTC::ExifUtilsWriter->new($main::EU_OPTION_PRAEFIX);
}

sub _print_info {
	my ($what) = @_;
    $main::log = Util::Logfile->new() if not defined $main::log;
    $main::log->log_print( "Benutze zum $what "
        . ( $main::USE_EXIFTOOL ? "ExifTool" : "die EXIFutils" ));
}

sub _check_globals {
    $main::log = Util::Logfile->new() if not defined $main::log;
    $main::log->fail('Variable $main::USE_EXIFTOOL ist nicht definiert!')
        if not defined $main::USE_EXIFTOOL;
    $main::log->fail('Variable $main::EU_OPTION_PRAEFIX ist nicht definiert!')
        if !$main::USE_EXIFTOOL && !defined $main::EU_OPTION_PRAEFIX;
}

1;

__END__

=head1 NAME

IPTC::Factory - Erzeugt einen passenden IPTC-Reader und IPTC-Writer

=head1 DESCRIPTION

Die Klasse C<IPTC::Factory> erzeugt einen passenden IPTC-Reader und
IPTC-Writer.

Jeder Reader implementiert C<read_file($filename)> und liefert einen
Hash mit IPTC-Daten durch C<get_iptc_data()>.

Jeder Writer implementiert C<write_file($filename, \%iptc_hash)>.

=head1 METHODS

=over

=item $iptc_reader = ITPC::Factory::create_reader()

Erzeugt einen Reader anhand der Vorgaben in der Konfigurationsdatei.

Der Rckgabewert ist eine Referenz auf den Reader

=item $iptc_writer = ITPC::Factory::create_writer()

Erzeugt einen Writer anhand der Vorgaben in der Konfigurationsdatei

Der Rckgabewert ist eine Referenz auf den Writer

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/25

=cut
