package Util::Array;

use version; our $VERSION = qv('0.2_0');
use strict;
use List::Util qw(first);

use Util::String;

sub get_hash_with_key_in_hash_array {
    my ( $array_ref, $key ) = @_;

    return undef if not ref $array_ref;
    return undef if ( not defined $array_ref || not defined $key );
    return undef if not ref($array_ref) eq 'ARRAY';

    foreach my $hash_ref ( @{$array_ref} ) {
        return $hash_ref if $hash_ref->{$key};
    }
    return undef;
}

sub elements_equals {
    my ( $array1_ref, $array2_ref ) = @_;

    return 1
        if  ( ( not defined $array1_ref ) && ( not defined $array2_ref ) );
    return 0 if (!ref $array1_ref || !ref $array2_ref);
    return 1 if ($array1_ref == $array2_ref);

    my $count1 = @{$array1_ref};
    my $count2 = @{$array2_ref};

    return 0 if $count1 != $count2;

    foreach my $element_in_1 ( @{$array1_ref} ) {
        my $element = first { $_ eq $element_in_1 } @{$array2_ref};
        return 0 if not defined $element;
    }
    return 1;
}

sub trim_all_strings {
    my ($array_ref) = @_;

    return if not defined $array_ref;
    return if not ref $array_ref;

    my $elment_count = @{$array_ref};

    for ( my $index = 0; $index < $elment_count; $index++ ) {
        Util::String::trim( \@{$array_ref}[$index] );
    }
}

sub exists {
    my ( $array, $element, $ignoreCase ) = @_;
    my $elementCount = @{$array};

    return 0 if not defined $element;

    for ( my $i = 0; $i < $elementCount; $i++ ) {
        if ($ignoreCase) {
            ( lc( $array->[$i] ) eq lc($element) ) && return 1;
        }
        else {
            ( $array->[$i] eq $element ) && return 1;
        }
    }

    return 0;
}

1;

__END__

=head1 NAME

Util::Array - Utilities fr Arrays

=head1 DESCRIPTION

Die Klasse Util::Array bietet Funktionen an fr die Arbeit mit Arrays.

=head1 METHODS

=over

=item $hash_ref = Util::Array::get_hash_with_key_in_hash_array(\@array_ref, $key)

Liefert die Referenz des ersten Hashes mit einem bestimmten Key in einem
Array mit Referenzen auf Hashes oder undef, falls nicht gefunden.

Der 1. Parameter ist eine Referenz auf den Array mit Hashreferenzen

Der 2. Parameter ist ein String mit dem Key

=item $equals = Util::Array::elements_equals(\@array1_ref, \@array2_ref)

Liefert, ob alle Elemente in einem Array gleich sind mit denen in einem
anderen. Ignoriert dabei die Reihenfolge und benutzt zum Vergleich C<eq>.
Beispielsweise ist der Array ['Heute', 'ist', 'Samstag'] gleich mit
['Samstag', 'ist' 'Heute']. 

Der 1. Parameter ist eine Referenz auf den 1. Array

Der 2. Parameter ist eine Referenz auf den 2. Array

Rckgabe: true, wenn die Elemente gleich sind

=item trim_all_strings(\@array_ref)

Trimmt in einem Array alle Strings.

Der 1. Parameter ist eine Referenz auf einen Array mit String-Elementen

=item $exists = Util::Array::exists( \@array, $element, $ignoreCase )

Liefert, ob in einem Array ein bestimmtes Element existiert. Nimmt zum Vergleich
den Operator C<eq>.

Der 1. Parameter ist ein Zeiger auf das Array.

Der 2. Parameter ist das Element, ein String.

Ist der 3. Parameter 0, wird zwischen Gro- und Kleinschreibung unterschieden.
Ist er 1, wird nicht zwischen Gro- und Kleinschreibung unterschieden.

Liefert 1, wenn das Element im Array ist, sonst 0.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/05/17

=cut
