package Util::System;

use version; our $VERSION = qv('0.1_0');

sub execute {
    my $command = shift;

    open PIPE, "$command 2>&1 |"
        or die "Util::System::execute(): Opening pipe failed: $!";
    my $output = <PIPE>;
    close PIPE;

    return $output;
}

sub execute_a {
    my $command = shift;

    open PIPE, "$command 2>&1 |"
        or die "Util::System::execute_a(): Opening pipe failed: $!";
    my @output = <PIPE>;
    close PIPE;

    return @output;
}

sub execute_check_error {
    my ($command, $error_string, $get_output_array_ref) = @_;
    my @output_lines = Util::System::execute_a($command);

    if ($get_output_array_ref && ref $get_output_array_ref) {
        @{$get_output_array_ref} = @output_lines;
    }
    
    foreach my $line (@output_lines) {
        my $error = ( (lc $line) =~ /error/ );
        return 0 if ($error);
    }
    return 1;
}


1;

__END__

=head1 NAME

Util::System - Verschiedene Hilfsfunktionen fr Systemaufrufe

=head1 DESCRIPTION

Bietet verschiedene Hilfsfunktionen fr Systemaufrufe an

=head1 METHODS

=over

=item $string = Util::System::execute( $command )

Fhrt auf dem System ein Kommando aus und liefert die erste Zeile dessen Outputs
als String

Der 1. Parameter ist ein String mit dem Kommando

=item @array = Util::System::execute_a($command)

Fhrt auf dem System ein Kommando aus und liefert dessen Output als
Array; Benutzung, wenn die Ausgabe mehrzeilig ist

Der 1. Parameter ist ein String mit dem Kommando

=item $success = Util::System::execute_check_error($command, $error_string, $get_output_array_ref)

Fhrt auf dem Betriebssystem ein Kommando aus und liefert den Erfolg anhand
des (Nicht-) Vorhandenseins eines Fehlerstrings. Benutzung, wenn eine Anwendung
stets den gleichen Rckgabewert hat, unabhngig vom Erfolg seiner Ausfhrung.

Der 1. Parameter ist ein String mit dem Kommando

Der 2. Parameter ist ein String, der einen Fehler bedeutet, falls er in der
Ausgabe des Programms auftaucht. Gro- bzw. Kleinschreibung wird ignoriert.

Der 3. Parameter ist optional und eine Referenz auf einen Array, der die
Ausgabe des Kommandos erhlt, falls definiert

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2006/10/07

=cut
