package XMP::ImageFileFinder;

use version; our $VERSION = qv('0.1_0');
use strict;
use File::Find;

use Util::File;
use Util::Array;

sub new {
	my ( $class, $xmp_filename_extension ) = @_;
	my $self = {};

	bless $self, $class;
	$self->{xmp_filename_extension} =
	  $xmp_filename_extension ? lc $xmp_filename_extension : 'xmp';

	return $self;
}

sub get_imagefilenames {
	my ($self) = @_;

	return @{ $self->{found_imagefiles_ref} };
}

sub find_imagefiles {
	my ( $self, $search_directories_ref, $image_filename_extensions_ref ) = @_;
	my $callback_function = sub { $self->_file_found(@_) };

	$self->{image_filename_extensions_ref} = $image_filename_extensions_ref;
	@{ $self->{found_imagefiles_ref} } = ();

	find( $callback_function, @{$search_directories_ref} );
}

sub _file_found {
	my ( $self, )          = @_;
	my $filename           = $File::Find::name;
	my $filename_extension = lc $filename;

	$filename_extension =~ s/.*\.//;
	if ( $filename_extension eq $self->{xmp_filename_extension} ) {
		$self->_set_image_filenames_for_xmp_file($filename);
	}
}

sub _set_image_filenames_for_xmp_file {
	my ( $self, $xmp_filename ) = @_;
	my $filename_without_extension = $xmp_filename;

	$filename_without_extension =~ s/(.*\.).*/$1/;
	foreach
	  my $filename_extension ( @{ $self->{image_filename_extensions_ref} } )
	{
		my $filename = "$filename_without_extension$filename_extension";
		if ( -f $filename ) {
			push @{ $self->{found_imagefiles_ref} }, $filename;
		}
	}
}

1;

__END__

=head1 NAME

XMP::ImageFileFinder - Findet fr XMP-Dateien Bilddateien

=head1 DESCRIPTION

Ein C<XMP::ImageFileFinder>-Objekt findet fr XMP-Dateien Bilddateien, sofern
diese den gleichen Namen haben wie die XMP-Datei, aber eine andere Endung.
Heit eine XMP-Datei C<Bild.xmp> knnten die Bilddateien C<Bild.tif> und/oder
C<Bild.jpg> heien.

=head1 METHODS

=over

=item $finder = XMP::ImageFileFinder->new($xmp_filename_extension);

Erzeugt ein neues Objekt.

Der 1. Parameter ist optional ein String mit der Endung der XMP-Datei;
Default: B<xmp>

=item $finder->find_imagefiles(\@search_directories, \@image_filename_extensions)

Findet im gleichen Verzeichnis alle Bilddateien zu einer XMP-Datei.

Der 1. Parameter ist eine Referenz auf ein String-Array mit den zu durchsuchenden
Verzeichnissen, jeder String ist ein Verzeichnispfad. Durchsucht werden diese
Verzeichnisse und deren Unterverzeichnisse.

Der 2. Parameter ist ein String-Array mit den Erweiterungen der Bilddateinamen,
die gesucht werden sollen, beispielsweise B<tif> oder B<jpg>

=item @filenames = $finder->get_imagefilenames()

Liefert ein String-Array mit allen gefundenen Bilddateinamen.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/17

=cut
