#!/usr/bin/perl

use version; our $VERSION = qv('0.1_5');
use Readonly;

# 'Path' ist relativ zu 'rdf:RDF -> rdf:Description'

Readonly %main::XMP_INFO => (
    'exif:DateTimeOriginal' => {
        Path      => ['exif:DateTimeOriginal'],
        ValueType => 'Date'
    },
    'dc:subject' => {
        Path       => [ 'dc:subject', 'rdf:Bag', 'rdf:li' ],
        ValueType  => 'bag Text',
        Repeatable => 1
    },
    'dc:creator' => {
        Path       => [ 'dc:creator', 'rdf:Seq', 'rdf:li' ],
        ValueType  => 'seq ProperName',
        Repeatable => 1
    },
    'dc:rights' => {
        Path       => [ 'dc:rights', 'rdf:Alt', 'rdf:li', 'content' ],
        ValueType  => 'Lang Alt',
        Repeatable => 1
    },
    'dc:description' => {
        Path       => [ 'dc:description', 'rdf:Alt', 'rdf:li', 'content' ],
        ValueType  => 'Lang Alt',
        Repeatable => 1
    },
    'dc:title' => {
        Path       => [ 'dc:title', 'rdf:Alt', 'rdf:li', 'content' ],
        ValueType  => 'Lang Alt',
        Repeatable => 1
    },
    'Iptc4xmpCore:Location' => {
        Path      => ['Iptc4xmpCore:Location'],
        ValueType => 'Text'
    },
    'Iptc4xmpCore:CountryCode' => {
        Path      => ['Iptc4xmpCore:CountryCode'],
        ValueType => 'Text'
    },
    'photoshop:Headline' => {
        Path      => ['photoshop:Headline'],
        ValueType => 'Text'
    },
    'photoshop:CaptionWriter' => {
        Path      => ['photoshop:CaptionWriter'],
        ValueType => 'ProperName'
    },
    'photoshop:Category' => {
        Path      => ['photoshop:Category'],
        ValueType => 'Text'
    },
    'photoshop:SupplementalCategories' => {
        Path => [ 'photoshop:SupplementalCategories', 'rdf:Bag', 'rdf:li' ],
        ValueType  => 'bag Text',
        Repeatable => 1
    },
    'photoshop:City' => {
        Path      => ['photoshop:City'],
        ValueType => 'Text'
    },
    'photoshop:State' => {
        Path      => ['photoshop:State'],
        ValueType => 'Text'
    },
    'photoshop:Country' => {
        Path      => ['photoshop:Country'],
        ValueType => 'Text'
    },
    'photoshop:TransmissionReference' => {
        Path      => ['photoshop:TransmissionReference'],
        ValueType => 'Text'
    },
    'photoshop:Instructions' => {
        Path      => ['photoshop:Instructions'],
        ValueType => 'Text'
    },
    'photoshop:Credit' => {
        Path      => ['photoshop:Credit'],
        ValueType => 'Text'
    },
    'photoshop:Source' => {
        Path      => ['photoshop:Source'],
        ValueType => 'Text'
    },
    'photoshop:AuthorsPosition' => {
        Path      => ['photoshop:AuthorsPosition'],
        ValueType => 'Text'
    },
);

Readonly %main::IPTC_INFO => (
    '2:05 Object Name' => {
        MaxBytes  => 64,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-object',
        ExifTool  => 'ObjectName'
    },
    '2:15 Category' => {
        MaxBytes   => 3,
        ValueType  => 'alphabetic characters',
        Deprecated => 1,
        ExifUtils  => 'ip-category',
        ExifTool   => 'Category'
    },
    '2:20 Supplemental Category' => {
        MaxBytes   => 32,
        ValueType  => 'graphic characters plus spaces',
        Repeatable => 1,
        Deprecated => 1,
        ExifUtils  => 'ip-suppcat',
        ExifTool   => 'SupplementalCategories'
    },
    '2:25 Keywords' => {
        MaxBytes   => 64,
        ValueType  => 'graphic characters plus spaces',
        Repeatable => 1,
        ExifUtils  => 'ip-keyword',
        ExifTool   => 'Keywords'
    },
    '2:26 Content Location Code' => {
        MinBytes   => 3,
        MaxBytes   => 3,
        ValueType  => 'alphabetic characters',
        Repeatable => 1,
        ExifUtils  => 'ip-cont-loc-code',
        ExifTool   => 'ContentLocationCode'
    },
    '2:27 Content Location Name' => {
        MaxBytes   => 64,
        ValueType  => 'graphic characters plus spaces',
        Repeatable => 1,
        ExifUtils  => 'ip-cont-loc-name',
        ExifTool   => 'ContentLocationName'
    },
    '2:40 Special Instructions' => {
        MaxBytes  => 256,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-instructions',
        ExifTool  => 'SpecialInstructions'
    },
    '2:62 Digital Creation Date' => {
        MaxBytes  => 8,
        MinBytes  => 8,
        ValueType => 'numeric characters',
        ExifUtils => 'ip-date',
        ExifTool  => 'DateCreated'
    },
    '2:63 Digital Creation Time' => {
        MaxBytes  => 11,
        MinBytes  => 11,
        ValueType => 'graphic characters',
        ExifUtils => 'ip-time',
        ExifTool  => 'TimeCreated'
    },
    '2:80 By-line' => {
        MaxBytes   => 32,
        ValueType  => 'graphic characters plus spaces',
        Repeatable => 1,
        ExifUtils  => 'ip-byline',
        ExifTool   => 'By-line'
    },
    '2:85 By-line Title' => {
        MaxBytes   => 32,
        ValueType  => 'graphic characters plus spaces',
        Repeatable => 1,
        ExifUtils  => 'ip-byline-title',
        ExifTool   => 'By-lineTitle'
    },
    '2:90 City' => {
        MaxBytes  => 32,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-city',
        ExifTool  => 'City'
    },
    '2:95 Province/State' => {
        MaxBytes  => 32,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-state-prov',
        ExifTool  => 'Province-State'
    },
    '2:101 Country/Primary Location Name' => {
        MaxBytes  => 64,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-country',
        ExifTool  => 'Country-PrimaryLocationName'
    },
    '2:103 Original Transmission Reference' => {
        MaxBytes  => 32,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-orig-ref',
        ExifTool  => 'OriginalTransmissionReference'
    },
    '2:105 Headline' => {
        MaxBytes  => 256,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-headline',
        ExifTool  => 'Headline'
    },
    '2:110 Credit' => {
        MaxBytes  => 32,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-credit',
        ExifTool  => 'Credit'
    },
    '2:115 Source' => {
        MaxBytes  => 32,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-source',
        ExifTool  => 'Source'
    },
    '2:116 Copyright Notice' => {
        MaxBytes  => 128,
        ValueType => 'graphic characters plus spaces',
        ExifUtils => 'ip-copyright',
        ExifTool  => 'CopyrightNotice'
    },
    '2:120 Caption/Abstract' => {
        MaxBytes => 2000,
        ValueType =>
            'graphic characters plus carriage-returns, linefeeds and spaces',
        ExifUtils => 'ip-caption',
        ExifTool  => 'Caption-Abstract'
    },
    '2:122 Writer/Editor' => {
        MaxBytes   => 32,
        ValueType  => 'graphic characters plus spaces',
        Repeatable => 1,
        ExifUtils  => 'ip-cap-writer',
        ExifTool   => 'Writer-Editor'
    },
);

Readonly %main::IPTC_KEY_FOR_XMP_KEY => (
    'dc:title'                         => '2:05 Object Name',
    'dc:creator'                       => '2:80 By-line',
    'dc:rights'                        => '2:116 Copyright Notice',
    'dc:description'                   => '2:120 Caption/Abstract',
    'dc:subject'                       => '2:25 Keywords',
    'Iptc4xmpCore:Location'            => '2:27 Content Location Name',
    'Iptc4xmpCore:CountryCode'         => '2:26 Content Location Code',
    'photoshop:Category'               => '2:15 Category',
    'photoshop:SupplementalCategories' => '2:20 Supplemental Category',
    'photoshop:Instructions'           => '2:40 Special Instructions',
    'photoshop:AuthorsPosition'        => '2:85 By-line Title',
    'photoshop:City'                   => '2:90 City',
    'photoshop:State'                  => '2:95 Province/State',
    'photoshop:Country'                => '2:101 Country/Primary Location Name',
    'photoshop:Headline'               => '2:105 Headline',
    'photoshop:Credit'                 => '2:110 Credit',
    'photoshop:Source'                 => '2:115 Source',
    'photoshop:CaptionWriter'          => '2:122 Writer/Editor',
    'photoshop:TransmissionReference' =>
        '2:103 Original Transmission Reference',
    'exif:DateTimeOriginal' =>
        [ '2:62 Digital Creation Date', '2:63 Digital Creation Time' ],
);

sub main::get_max_bytes_for_iptc_value {
    my ($iptc_key) = @_;

    return undef if not defined $main::IPTC_INFO{$iptc_key};

    return $main::IPTC_INFO{$iptc_key}->{MaxBytes};
}

sub main::is_iptc_value_repeatable {
    my ($iptc_key) = @_;

    return undef if not defined $main::IPTC_INFO{$iptc_key};

    return $main::IPTC_INFO{$iptc_key}->{Repeatable};
}

sub main::is_xmp_value_repeatable {
    my ($xmp_key) = @_;

    return undef if not defined $main::XMP_INFO{$xmp_key};

    return $main::XMP_INFO{$xmp_key}->{Repeatable};
}

sub main::has_xmp_key_multiple_iptc_keys {
    my ($xmp_key) = @_;

    return undef if not defined $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key};

    return ref $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key}
        && ref $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key} eq 'ARRAY';
}

1;

__END__

=head1 NAME

globals - Globale Konstanten und Zugriffsfunktionen auf diese

=head1 VARIABLES

=over

=item %main::XMP_INFO

Hash mit Informationen ber XMP-Werte.

Auflistung der Schlssel, Spalte B<XMP>:
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/xmp-iptc-mapping.html>

XMP-Spezifikation: L<http://www.adobe.com/devnet/xmp/pdfs/xmp_specification.pdf>

Jeder Schlssel im Hash verweist auf einen Hash mit folgenden Informationen:

=over

=item Path

Pfad relativ zu 'rdf:RDF -> rdf:Description' zum XMP-Wert in einer XMP-Datei

=item ValueType

Typ des XMP-Werts gem XMP-Spezifikation

=item Repeatable

Nur definiert, wenn der XMP-Wert sich wiederholen kann; dann ist er true

=back

=item %main::IPTC_INFO

Ein Hash mit allen bercksichtigten IPTC-Schlsseln und Informationen darber. 

Eine Liste der Schlssel ist unter
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/xmp-iptc-mapping.html>,
Spalte B<IPTC>.

IPTC-Standard: L<http://www.iptc.org/IIM/4.1/specification/IIMV4.1.pdf>

Jedem Schlssel ist ein Hash zugeordnet mit folgenden Informationen:

=over

=item MinBytes

Minimale Anzahl an Bytes, die der IPTC-Wert haben muss. Ist nur definiert,
falls eine minimale Byteanzahl vorgeschrieben ist.

=item MaxBytes

Maximale Anzahl an Bytes, die der IPTC-Wert haben darf

=item ValueType

Typ des IPTC-Werts gem IPTC-Spezifikation

=item Repeatable

True, falls der IPTC-Wert sich wiederholen kann. Ist nur definiert, falls
der IPTC-Wert sich wiederholen kann.

=item Deprecated

True, wenn die Nutzung dieses IPTC-Werts missbilligt ist. Ist nur in diesem
Fall definiert.

=item ExifUtils

Option fr die EXIFutils (L<http://www.hugsan.com/EXIFutils/>) fr den
Zugriff auf diesen IPTC-Wert

=item ExifTool

Schlssel fr ExifTool (L<http://www.sno.phy.queensu.ca/~phil/exiftool/>)
fr den Zugriff auf diesen IPTC-Wert

=back

=item %main::IPTC_KEY_FOR_XMP_KEY

Ordnet jedem XMP-Schlssel den IPTC-Schlssel mit gleicher Bedeutung
zu. Somit steht fest, in welches IPTC-Feld ein XMP-Wert geschrieben wird.
Liste aller XMP- und IPTC-Schlssel, Spalten B<XMP> und B<IPTC>:
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/xmp-iptc-mapping.html>

Verteilt sich ein XMP-Wert auf mehrere IPTC-Werte, ist der Wert ein
Array mit allen IPTC-Schlsseln, auf denen sich der XMP-Wert verteilt.

=back

=head1 METHODS

=over

=item $max_bytes = main::get_max_bytes_for_iptc_value($iptc_key)

Liefert die maximale Byteanzahl, die ein IPTC-Wert haben darf.

Der 1. Parameter ist ein String mit dem IPTC-Schlssel

Der Rckgabewert ist die Byteanzahl

=item $is_repeatable = main::is_iptc_value_repeatable($iptc_key)

Liefert, ob sich ein IPTC-Wert wiederholen kann.

Der 1. Parameter ist ein String mit dem IPTC-Schlssel

Der Rckgabewert ist true, wenn sich der IPTC-Wert wiederholen kann

=item $is_repeatable = main::is_xmp_value_repeatable($xmp_key)

Liefert, ob sich ein XMP-Wert wiederholen kann.

Der 1. Parameter ist ein String mit dem XMP-Schlssel

Der Rckgabewert ist true, wenn sich der XMP-Wert wiederholen kann

=item $has_multiple_keys = main::has_xmp_key_multiple_iptc_keys($xmp_key)

Liefert, ob ein XMP-Wert auf mehrere IPTC-Werte verteilt ist.

Der 1. Parameter ist ein String mit dem XMP-Schlssel

Der Rckgabewert ist true, wenn der XMP-Wert sich auf mehrere
IPTC-Werte verteilt

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/25

=cut
