#!/usr/bin/perl -w

use strict;
use Test::More qw( no_plan );

use IPTC::Factory;
use Util::Array;

require 'globals.pl';
if ( defined $ENV{XMP_IPTC_CONFIG_FILENAME} ) {
    require $ENV{XMP_IPTC_CONFIG_FILENAME};
}

my $RES_DIR = "$ENV{PROJECTDIR}/XmpIptc/t/images";

my %FILE_DATA = (
    "$RES_DIR/iptc_alles.tif" => {
        '2:25 Keywords'              => ['keyword'],
        '2:27 Content Location Name' => ['content location name'],
        '2:62 Digital Creation Date' => '20071013',
        '2:63 Digital Creation Time' => '025500+0200',
        '2:80 By-line'               => ['byline'],
        '2:105 Headline'             => 'headline',
        '2:116 Copyright Notice'     => 'copyright notice',
        '2:120 Caption/Abstract'     => 'caption',
    },
    "$RES_DIR/iptc_alles_umlaute_iso_8859_1.tif" => {
        '2:25 Keywords'              => ['keyword '],
        '2:27 Content Location Name' => ['content location name '],
        '2:62 Digital Creation Date' => '20071013',
        '2:63 Digital Creation Time' => '025900+0200',
        '2:80 By-line'               => ['byline '],
        '2:105 Headline'             => 'headline ',
        '2:116 Copyright Notice'     => 'copyright notice ',
        '2:120 Caption/Abstract'     => 'caption ',
    },
    "$RES_DIR/iptc_nur_byline.tif" => { '2:80 By-line' => ['byline'] },
    "$RES_DIR/iptc_nur_caption.tif" =>
        { '2:120 Caption/Abstract' => 'caption' },
    "$RES_DIR/iptc_nur_content_location_name.tif" =>
        { '2:27 Content Location Name' => ['content location name'] },
    "$RES_DIR/iptc_nur_copyright_notice.tif" =>
        { '2:116 Copyright Notice' => 'copyright notice' },
    "$RES_DIR/iptc_nur_creation_date.tif" =>
        { '2:62 Digital Creation Date' => '20071013' },
    "$RES_DIR/iptc_nur_creation_time.tif" =>
        { '2:63 Digital Creation Time' => '024600+0200' },
    "$RES_DIR/iptc_nur_headline.tif"   => { '2:105 Headline' => 'headline' },
    "$RES_DIR/iptc_nur_keywords_1.tif" => { '2:25 Keywords'  => ['keyword'] },
    "$RES_DIR/iptc_nur_keywords_2.tif" =>
        { '2:25 Keywords' => [ 'keyword 1', 'keyword 2' ] },
    "$RES_DIR/iptc_ohne.jpg"     => {},
    "$RES_DIR/iptc_komplett.jpg" => {
        '2:05 Object Name'                      => 'Bezeichnung',
        '2:15 Category'                         => 'Kat',
        '2:20 Supplemental Category'            => ['Andere Kategorien'],
        '2:25 Keywords'                         => [ 'Herbst', 'Landschaft' ],
        '2:26 Content Location Code'            => ['cl1'],
        '2:27 Content Location Name'            => ['Ort 1'],
        '2:40 Special Instructions'             => 'Anweisungen',
        '2:62 Digital Creation Date'            => '20071024',
        '2:63 Digital Creation Time'            => '181737+0200',
        '2:80 By-line'                          => ['eb@elmar-baumann.de'],
        '2:85 By-line Title'                    => ['Position'],
        '2:90 City'                             => 'Stadt',
        '2:95 Province/State'                   => 'Bundesland',
        '2:101 Country/Primary Location Name'   => 'Land',
        '2:103 Original Transmission Reference' => 'Auftragskennung',
        '2:105 Headline'                        => 'IPTC-Titel',
        '2:110 Credit'                          => 'Anbieter',
        '2:115 Source'                          => 'Quelle',
        '2:116 Copyright Notice' => 'Copyright (c) 2007 Elmar Baumann',
        '2:120 Caption/Abstract' => 'Bildbeschreibung',
        '2:122 Writer/Editor'    => ['Autor der Beschreibung'],
    },
);

test( IPTC::Factory::create_reader() );

sub test {
    my ($reader) = @_;
    print "Teste $reader\n";
    foreach my $filename ( keys %FILE_DATA ) {
        $reader->read_file($filename);
        my $read_hash_ref     = $reader->get_iptc_data();
        my $expected_hash_ref = $FILE_DATA{$filename};
        foreach my $iptc_key ( keys %main::IPTC_INFO ) {
            my $test_name           = "$filename [$iptc_key]";
            my $read_iptc_value     = $read_hash_ref->{$iptc_key};
            my $expected_iptc_value = $expected_hash_ref->{$iptc_key};
            if ( main::is_iptc_value_repeatable($iptc_key) ) {
                ok( Util::Array::elements_equals(
                        $read_iptc_value, $expected_iptc_value
                    ),
                    $test_name
                );
            }
            else {
                is( $read_iptc_value, $expected_iptc_value, $test_name );
            }
        }
    }
}
