#!/usr/bin/perl -w

use strict;
use Test::More qw( no_plan );
use File::Copy;

use IPTC::Factory;
use Util::Array;

require 'globals.pl';

if ( defined $ENV{XMP_IPTC_CONFIG_FILENAME} ) {
    require $ENV{XMP_IPTC_CONFIG_FILENAME};
}

my $RES_DIR            = "$ENV{PROJECTDIR}/XmpIptc/t/images";
my $TEST_TEMP_FILENAME = "$RES_DIR/iptc_writer_test_temp.jpg";

my %FILE_DATA = (
    "$RES_DIR/iptc_ohne.jpg" => {
        '2:05 Object Name' => 'object name ',
        '2:15 Category'    => 'cat',
        '2:20 Supplemental Category' =>
            [ 'supplemental cat 1', 'supplemental cat 2' ],
        '2:25 Keywords' =>
            [ 'keyword 1 ', 'keyword 2', 'keyword 3' ],
        '2:26 Content Location Code'            => ['clc'],
        '2:27 Content Location Name'            => ['cln'],
        '2:40 Special Instructions'             => 'special instructions',
        '2:62 Digital Creation Date'            => '20071025',
        '2:63 Digital Creation Time'            => '215200+0200',
        '2:80 By-line'                          => ['By line'],
        '2:85 By-line Title'                    => ['By line title'],
        '2:90 City'                             => 'city',
        '2:95 Province/State'                   => 'state',
        '2:101 Country/Primary Location Name'   => 'country location name',
        '2:103 Original Transmission Reference' => 'original transm. reference',
        '2:105 Headline'                        => 'headline',
        '2:110 Credit'                          => 'credit',
        '2:115 Source'                          => 'source',
        '2:116 Copyright Notice'                => 'copyright notice',
        '2:120 Caption/Abstract'                => 'caption abstract',
        '2:122 Writer/Editor'                   => [ 'writer 1', 'writer 2' ],
    },
    "$RES_DIR/iptc_komplett.jpg" => {
        '2:05 Object Name' => 'object name ',
        '2:25 Keywords' =>
            [ 'keyword 1 ', 'keyword 2', 'keyword 3' ],
    },
    "$RES_DIR/iptc_komplett.jpg" => {},
);

test();

sub test {
    my $writer = IPTC::Factory::create_writer();
    my $reader = IPTC::Factory::create_reader();

    print "Teste $writer, Lesen durch $reader\n";

    foreach my $filename ( keys %FILE_DATA ) {
        my $iptc_hash_ref = $FILE_DATA{$filename};
        print "Testdatei: $filename\n";
        copy_test_file($filename);
#        die if $filename eq '/mnt/daten/projekte/XmpIptc/t/images/iptc_writer_test_temp.jpg';
        die "Fehler beim Schreiben von Testdatei '$filename'!"
            if not $writer->write_file( $TEST_TEMP_FILENAME, $iptc_hash_ref );
        $reader->read_file($TEST_TEMP_FILENAME);
        my $read_iptc_hash_ref = $reader->get_iptc_data();
        foreach my $iptc_key ( keys %main::IPTC_INFO ) {
            my $expected_value = $iptc_hash_ref->{$iptc_key};
            my $read_value     = $read_iptc_hash_ref->{$iptc_key};
            my $test_name      = "$TEST_TEMP_FILENAME [$iptc_key] -> "
                . ( $expected_value ? "'$expected_value'" : 'undef' ) . " == "
                . ( $read_value     ? "'$read_value'"     : 'undef' );
            if ( main::is_iptc_value_repeatable($iptc_key) ) {
                ok( Util::Array::elements_equals(
                        $read_value, $expected_value
                    ),
                    $test_name
                );
            }
            else {
                is( $read_value, $expected_value, $test_name );
            }
        }
        unlink $TEST_TEMP_FILENAME;
    }
}

sub copy_test_file {
    my ($source_filename) = @_;

    if ( -f $TEST_TEMP_FILENAME ) {
        unlink $TEST_TEMP_FILENAME;
    }
    copy( $source_filename, $TEST_TEMP_FILENAME )
        or die
        "Kopieren von '$source_filename' nach '$TEST_TEMP_FILENAME ' ist fehlgeschlagen!";
}
