#!/usr/bin/perl -w

use strict;
use Test::More qw( no_plan );

use Util::Array;

# Util::Array::exists() aus anderem Projekt, dort getestet

test_get_hash_with_key_in_hash_array();
test_elements_equals();
test_trim_all_strings();

sub test_get_hash_with_key_in_hash_array {
    my %hash_1 = ( kamera   => 'kamera' );
    my %hash_2 = ( objektiv => 'objektiv' );
    my %hash_3 = ( blitzgeraet => 'blitzgeraet', batterie => 'batterie' );

    my @array_1 = ( \%hash_1, \%hash_2, \%hash_3 );
    my @array_2 = ();
    my @array_3;

    is( Util::Array::get_hash_with_key_in_hash_array( \@array_1, 'kamera' ),
        \%hash_1 );
    is( Util::Array::get_hash_with_key_in_hash_array( \@array_1, 'objektiv' ),
        \%hash_2 );
    is( Util::Array::get_hash_with_key_in_hash_array( \@array_1, 'batterie' ),
        \%hash_3 );
    is( Util::Array::get_hash_with_key_in_hash_array( \@array_1, 'sensor' ),
        undef );
    is( Util::Array::get_hash_with_key_in_hash_array( \@array_2, 'batterie' ),
        undef );
    is( Util::Array::get_hash_with_key_in_hash_array( \@array_3, 'batterie' ),
        undef );    
}

sub test_elements_equals {
    my @array_4  = ( "Eins", "Zwei", "Drei" );
    my @array_5  = ();
    my @array_6  = ();
    my @array_7  = ( "Drei", "Zwei", "Eins" );
    my @array_8  = ("");
    my @array_9  = ("");
    my @array_10 = ( "Eins", "Zwei", "Drei", "Vier" );
    my @array_11 = ("Elektronenblitzgert");
    my @array_12 = ("Elektronenblitzgert");

    ok( Util::Array::elements_equals( \@array_4,   \@array_7 ) );
    ok( Util::Array::elements_equals( \@array_5,   \@array_6 ) );
    ok( Util::Array::elements_equals( \@array_8,   \@array_9 ) );
    ok( Util::Array::elements_equals( \@array_11,  \@array_12 ) );
    ok( Util::Array::elements_equals( undef,       undef ) );
    ok( !Util::Array::elements_equals( \@array_7,  \@array_10 ) );
    ok( !Util::Array::elements_equals( \@array_5,  \@array_8 ) );
    ok( !Util::Array::elements_equals( \@array_10, \@array_11 ) );
}

sub test_trim_all_strings {
    my @string_array = (
        "\n\n\t\telement\n\n\t\t", "element\n\n\t\t",
        "\n\n\t\telement",         " element ",
        "element"
    );
    my $expected_element_count = @string_array;

    Util::Array::trim_all_strings( \@string_array );
    my $element_count = @string_array;

    is( $element_count, $expected_element_count );

    foreach my $element (@string_array) {
        is( $element, 'element' );
    }
}
